//модуль для клиента
'use strict';
// это клиент

// Зависимости
let timestamp = require('time-stamp');
//let nanoid = require('nanoid');
let axios = require('axios');
const jwt        = require('jsonwebtoken');

const logType = 'prod' // Prod - отсутствие логов, uat- только критические логи, dev - все логи (по умолчанию prod, если в start не передано иное)
let systemToken = null;
let baseUrl = null; // Адрес для базового запроса, получаемый из токена при start

function getHeaders(){
    const headers = {
        systemtoken: systemToken
    };
    if (localStorage.getItem("accesstoken") !== null){
        headers.accesstoken = localStorage.getItem("accesstoken");
    }
    log(`getHeaders возвращает headers: ${JSON.stringify(headers)}`, 'info');
    return headers
}

function getBaseUrl(){
    if (localStorage.getItem("baseUrl") !== null){
        log(`getBaseUrl возвращает базовый путь: ${localStorage.getItem("baseUrl")}`, 'info');
        return localStorage.getItem("baseUrl")
    } else {
        localStorage.setItem('baseUrl', baseUrl);
        log(`getBaseUrl возвращает базовый путь: ${baseUrl}`, 'info');
        return baseUrl
    }
}

async function checkAndRefreshTokens(mode){
    // сначала проверяем есть ли сохраненные токены
    log('Проверка токена и обновление при необходимости', 'info');
    const access_token_expireIn = localStorage.getItem("access_token_expireIn");
    const access_token = localStorage.getItem("accesstoken");
    const refresh_token = localStorage.getItem("refresh_token");

    const now = Math.floor(Date.now() / 1000);
    if ((access_token && (access_token_expireIn-now)>10) && (!mode || mode !== 'forced')){
        log('accesstoken ещё свежий. Обновление не требуется', 'info');
        return {result: 'ok', description: 'accesstoken ещё свежий. Обновление не требуется'}
        // Т.е. если токен есть и он не просрочен - добавляем в хедер
    } else if ((access_token && (access_token_expireIn-now)<10)|| (access_token && mode === 'forced')){
        // Если токен есть и он просрочен - обновляем пару токенов, сохраняем и новый access_token добавляем в хедер
        log('Обновление accesstoken', 'info');
        const reqParams = {
            refresh_token: refresh_token
        };
        try {
            const resp = await axios.put(
                getBaseUrl() + '/sgwc/apigw_fs/login',
                reqParams,
                {headers: {systemtoken: systemToken}}
            );
            if (resp.data.result === 'ok') {
                log('Произошло обновление токенов', 'info');
                localStorage.setItem('hasManyProfiles', resp.data.data.userData.hasManyProfiles);
                localStorage.setItem('userLogin', resp.data.data.userData.userLogin);
                localStorage.setItem('userEmail', resp.data.data.userData.userEmail);
                localStorage.setItem('urlUserAvatar', resp.data.data.userData.urlUserAvatar);
                localStorage.setItem('userProfile', resp.data.data.userData.userProfile.profileName);
                localStorage.setItem('accesstoken', resp.data.data.tokensData.access_token);
                localStorage.setItem('access_token_expireIn', resp.data.data.tokensData.expiresIn);
                localStorage.setItem('refresh_token', resp.data.data.tokensData.refresh_token);
                return {result: 'ok', description: 'Произошло обновление токенов'}
            } else {
                log('Рефреш-токен устарел. Осуществляется разлогирование', 'wrn');
                // значит сначала удаляем сторадж
                localStorage.removeItem('userLogin');
                localStorage.removeItem('userEmail');
                localStorage.removeItem('urlUserAvatar');
                localStorage.removeItem('userProfile');
                localStorage.removeItem('hasManyProfiles');
                localStorage.removeItem('accesstoken');
                localStorage.removeItem('access_token_expireIn');
                localStorage.removeItem('refresh_token');
                localStorage.removeItem('baseUrl');
                return {result: 'error', reason: 'Рефреш-токен устарел. Осуществляется разлогирование'}
            }
        } catch (e) {
            log('Ошибка обращения к серверу', 'error');
            return {result: 'error', reason: 'Ошибка обращения к серверу'}
            //this.$message.success(`Ошибка обращения к серверу (при получении токена)`);
        }
    } else {
        log('Токены отсутствуют (работа неавторизованного пользователя)');
        return {result: 'ok', description: 'Токены отсутствуют (работа неавторизованного пользователя)'}
    }
}

async function callGet (reqPath, reqParams) {
    if (localStorage.getItem('accesstoken')){
        let checkAndRefreshTokensResult = await checkAndRefreshTokens();
        if (checkAndRefreshTokensResult.result === 'error'){
            return {result: 'error', reason: checkAndRefreshTokensResult.reason, errorType: 'auth'}
        }
    }
    try {
        log(`Инициирование GET запроса к ${reqPath}`, 'info');
        const resp = await axios.get(
            getBaseUrl() + reqPath,
            {
                params:reqParams,
                headers: getHeaders()
            });
        return resp.data
    } catch (e) {
        log('Ошибка обращения к серверу', 'error');
        return {result: 'error', reason: 'Ошибка обращения к серверу'}
    }
}

async function callPost (reqPath, reqParams) {
    if (localStorage.getItem('accesstoken')){
        let checkAndRefreshTokensResult = await checkAndRefreshTokens();
        if (checkAndRefreshTokensResult.result === 'error'){
            return {result: 'error', reason: checkAndRefreshTokensResult.reason, errorType: 'auth'}
        }
    }
    try {
        const resp = await axios.post(
            getBaseUrl() + reqPath,
            reqParams,
            {headers: getHeaders()}
        );
        return resp.data
    } catch (e) {
        log('Ошибка обращения к серверу', 'error');
        return {result: 'error', reason: 'Ошибка обращения к серверу'}
    }
}

async function callPut (reqPath, reqParams) {
    if (localStorage.getItem('accesstoken')){
        let checkAndRefreshTokensResult = await checkAndRefreshTokens();
        if (checkAndRefreshTokensResult.result === 'error'){
            return {result: 'error', reason: checkAndRefreshTokensResult.reason, errorType: 'auth'}
        }
    }
    try {
        const resp = await axios.put(
            getBaseUrl() + reqPath,
            reqParams,
            {headers: getHeaders()}
        );
        return resp.data
    } catch (e) {
        log('Ошибка обращения к серверу', 'error');
        return {result: 'error', reason: 'Ошибка обращения к серверу'}
    }
}

async function callDel (reqPath, reqParams) {
    if (localStorage.getItem('accesstoken')){
        let checkAndRefreshTokensResult = await checkAndRefreshTokens();
        if (checkAndRefreshTokensResult.result === 'error'){
            return {result: 'error', reason: checkAndRefreshTokensResult.reason, errorType: 'auth'}
        }
    }
    try {
        const resp = await axios.delete(
            getBaseUrl() + reqPath,
            {
                params:reqParams,
                headers: getHeaders()
            });
        return resp.data
    } catch (e) {
        log('Ошибка обращения к серверу', 'error');
        return {result: 'error', reason: 'Ошибка обращения к серверу'}
    }
}

function log (logString, mark){
    let currentLogType = localStorage.getItem("logType") || logType
    if ((currentLogType === 'dev') || ((currentLogType === 'uat' && (mark === 'error') ||(mark === 'wrn')))) {
        const msMark = mark || 'ok' ;
        console.log(`${timestamp('[DD-MM-YYYY HH:mm:ss.ms]')}: (${msMark}) ${logString}` )
    }
    if (((currentLogType === 'dev') || (currentLogType === 'uat')) && (!mark)) {
        console.log(logString)
    }
}

async function login (loginParams){
    const reqParams = {
        userEmail: loginParams.userEmail, // Логин
        userPass: loginParams.userPass, // Пароль
    };
    const resp = await callGet('/sgwc/apigw_fs/login', reqParams);
    // this.loading = false;
    if (resp.result === 'ok'){

        log('Успешно выполнен вход пользователя', 'info')

        localStorage.setItem('hasManyProfiles', resp.data.userData.hasManyProfiles);
        localStorage.setItem('userLogin', resp.data.userData.userLogin);
        localStorage.setItem('userEmail', resp.data.userData.userEmail);
        localStorage.setItem('urlUserAvatar', resp.data.userData.urlUserAvatar);
        localStorage.setItem('userProfile', resp.data.userData.userProfile.profileName);
        localStorage.setItem('accesstoken', resp.data.tokensData.access_token);
        localStorage.setItem('access_token_expireIn', resp.data.tokensData.expiresIn);
        localStorage.setItem('refresh_token', resp.data.tokensData.refresh_token);

        //this.$router.push({name:'Home'});
        return {result: 'ok', description: 'Выполнен login пользователя', data: resp.data}
    }
    else {
        // Если не добавлен (что-то пошло не так)
        log('Ошибка выполнения входа пользователя', 'wrn')
        // this.$message.error(resp.data.reason);
        return {result: 'error', reason: resp.reason, subReason: resp.subReason}
    }


}

function currentUserParams (){
    if (localStorage.getItem('accesstoken')){
        const data = {
            hasManyProfiles: localStorage.getItem('hasManyProfiles') === 'true',
            userLogin: localStorage.getItem('userLogin'),
            userEmail: localStorage.getItem('userEmail'),
            urlUserAvatar: localStorage.getItem('urlUserAvatar'),
            userProfile: localStorage.getItem('userProfile'),
            access_token: localStorage.getItem('accesstoken'),
            profileId: jwt.decode(localStorage.getItem("accesstoken"), {complete: true}).payload.profileId
        }
        return {result: 'ok', description: 'Данные об авторизованном пользователе', data: data}
    } else {
        return {result: 'error', reason: 'Отсутствуют данные о пользователе'}
    }
}

async function getAvailableProfiles (){
    const resp = await callGet('/sgwc/apigw_fs/userProfiles'); // Входящие параметры не требуются, всё необходмое берется на бэке из токена фронт-системы
    if (resp.result === 'ok'){
        log('Успешное получение профилей', 'info')
        return {result: 'ok', description: 'Профили, доступные пользователю в текущей фронт-системе', data: resp.data}
    } else {
        log('Ошибка получения профилей', 'error')
        return {result: 'error', reason: resp.reason}
    }
}

async function changeProfile (chosenProfile){
    const refresh_token = localStorage.getItem("refresh_token");
    const reqParams = {refresh_token: refresh_token, chosenProfile: chosenProfile};
    log('Осуществлен запрос на смену профиля', 'info')
    const resp = await callPut('/sgwc/apigw_fs/login', reqParams);
    if (resp.result === 'ok'){
        localStorage.setItem('hasManyProfiles', resp.data.userData.hasManyProfiles);
        localStorage.setItem('userLogin', resp.data.userData.userLogin);
        localStorage.setItem('userEmail', resp.data.userData.userEmail);
        localStorage.setItem('urlUserAvatar', resp.data.userData.urlUserAvatar);
        localStorage.setItem('userProfile', resp.data.userData.userProfile.profileName);
        localStorage.setItem('accesstoken', resp.data.tokensData.access_token);
        localStorage.setItem('access_token_expireIn', resp.data.tokensData.expiresIn);
        localStorage.setItem('refresh_token', resp.data.tokensData.refresh_token);
        return {result: 'ok', description: 'Выполнена смена профиля пользователя'}
    } else {
        log('Ошибка смены профиля пользователя', 'error')
        return {result: 'error', reason: resp.reason}
    }
}

async function logout (){
    const reqParams = {
        refresh_token: localStorage.getItem('refresh_token')
    };
    log('Осуществлен запрос на выход из системы', 'info')
    let resp = await callDel('/sgwc/apigw_fs/login', reqParams);
    if (resp.result === 'ok'){
        localStorage.removeItem('userLogin');
        localStorage.removeItem('userEmail');
        localStorage.removeItem('userProfile');
        localStorage.removeItem('urlUserAvatar');
        localStorage.removeItem('hasManyProfiles');
        localStorage.removeItem('accesstoken');
        localStorage.removeItem('access_token_expireIn');
        localStorage.removeItem('refresh_token');
        localStorage.removeItem('baseUrl');
        return {result: 'ok', description: 'Выполнен выход из системы'}
    } else {
        log('Ошибка выхода из системы', 'error')
        return {result: 'error', reason: resp.reason}
    }
}

async function availabilityServer (){
    const resp = await callGet('/sgwc/global_provider/availability_server_for_fs');
        if (resp.result === 'error'){
            return {result: 'error', reason: resp.reason}
        } else {
            log('Успешное получение данных по доступным серверам (apigw_fs)', 'info')

            resp.data.availabilityServer.reduceRight(function (previousValue, currentValue) {
                currentValue.ping = 0
            }, 0);

            resp.data.availabilityServer.reduceRight(async function (previousValue, currentValue) {
                let time1 = Date.now()
                try {
                    await callGet('/sgwc/apigw_fs/ping');
                    let time2 = Date.now()
                    currentValue.ping = time2 - time1
                } catch (e) {
                    log('Ошибка обращения к серверу', 'error')
                    return {result: 'error', reason: resp.reason}
                }
            }, 0);
            return {result: 'ok', description: 'Доступные сервера apigw_fs', data: {availabilityServer: resp.data.availabilityServer}}
        }
}

function setBasePathApigwFs (basePathApigwFs){
    localStorage.setItem('baseUrl', basePathApigwFs);
}

async function registerUser (regParams){

    const reqParams = {
        userLogin: regParams.userLogin,
        userEmail: regParams.userEmail,
        userPass: regParams.userPass,
        invitationKey: regParams.invitationKey,
    };
    const resp = await callPost('/sgwi/reguserws/user', reqParams);
    if (resp.result === 'ok'){
        log('Успешная регистрация пользователя и отправка письма с ссылкой', 'info')
        return {result: 'ok', description: 'Успешная регистрация пользователя и отправка письма с ссылкой'}
    }
    else {
        // Если не добавлен (что-то пошло не так)
        log('Ошибка регистрации пользователя и отправки письма с ссылкой', 'error')
        return {result: 'error', reason: resp.reason}
    }
}

async function confirmEmailToken (confirmEmailToken){
    const reqParams = {
        confirmemailtoken: confirmEmailToken
    };
    const resp = await callPost('/sgwi/reguserws/confirmemail', reqParams);
    if (resp.result === 'ok') {
        log('Подтверждение Email адреса', 'info')
        return {result: 'ok', description: 'Успешное подтверждение Email адреса'}
    } else {
        log('Ошибка подтверждения Email адреса', 'error')
        return {result: 'error', reason: resp.reason}
    }
}


async function requestPasswordReset (userEmail){
    const reqParams = {
        userEmail: userEmail
    };
    const resp = await callPut('/sgwi/reguserws/recoveryUserPass', reqParams);

    if (resp.result === 'ok'){
        log('Успешно отправлено письмо с ссылкой для смены пароля', 'info')
        return {result: 'ok', description: 'Успешно отправлено письмо с ссылкой для смены пароля'}
    }
    else {
        log('Ошибка отправки письма с ссылкой для смены пароля', 'error')
        return {result: 'error', reason: resp.reason}
    }
}

async function confirmPasswordReset (tmpPassToken){
    const reqParams = {
        tmpPassToken: tmpPassToken
    };
    const resp = await callPost('/sgwi/reguserws/confirmTmpPass', reqParams);

    if (resp.result === 'ok') {
        log('Успешная смена пароля', 'info')
        return {result: 'ok', description: 'Успешная смена пароля'}
    } else {
        log('Ошибка смены пароля', 'error')
        return {result: 'error', reason: resp.reason}
    }
}


// function checkAndUpdateLogin (){
//
// }

function start (startParams){
    // startParams.systemToken
    // startParams.logType // Тип логирования (prod, uat, dev)
    // startParams.
    if (startParams && startParams.systemToken){
        systemToken = startParams.systemToken;
        baseUrl = jwt.decode(systemToken, {complete: true}).payload.baseUrl;
    } else {
        console.log('Не передан обязательный systemToken для применения модуля')
    }


    if (startParams && startParams.logType){
        localStorage.setItem('logType', startParams.logType);
    }

    console.log('Лог вызванный из подключаемого модуля')
}


module.exports = {
// Работа с запросами
    // Авторизация
    login: login, // С профилем по умолчанию или последним использованным профилем
    // Смена профиля
    changeProfile: changeProfile, // Изменение используемого профиля
    getAvailableProfiles: getAvailableProfiles,
    currentUserParams: currentUserParams,
    availabilityServer: availabilityServer,
    setBasePathApigwFs: setBasePathApigwFs,
    // Разлогирование
    logout: logout,
    // Проверка действия авторизации (по вреимени, возможно профиль)
    checkAndRefreshTokens: checkAndRefreshTokens,
    // Регистрация пользователя
    registerUser: registerUser,
    // Подтверждение почты пользователя
    confirmEmailToken: confirmEmailToken,
    // Запрос сброса пароля пользователя
    requestPasswordReset: requestPasswordReset,
    // Подтверждение сброса пароля пользователя
    confirmPasswordReset: confirmPasswordReset,
    // Для вызова апи
    callGet: callGet,
    callPost: callPost,
    callPut: callPut,
    callDel: callDel,
    // Метод при запуске системы (приложения)
    start: start,
    // Логирование, в соответствии с режимом работы фронт-системы
    log: log
};
